@@ -1,4 +1,4 @@
-/* $Id: buffer-poll.c,v 1.10 2008/09/09 22:16:36 nicm Exp $ */
+/* $Id: buffer-poll.c,v 1.18 2009/10/23 17:49:47 tcunha Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -23,39 +23,17 @@
 
 #include "tmux.h"
 
-/* Set up pollfd for buffers. */
-void
-buffer_set(
-    struct pollfd *pfd, int fd, unused struct buffer *in, struct buffer *out)
-{
-	pfd->fd = fd;
-	pfd->events = POLLIN;
-	if (BUFFER_USED(out) > 0)
-		pfd->events |= POLLOUT;
-}
-
 /* Fill buffers from socket based on poll results. */
 int
-buffer_poll(struct pollfd *pfd, struct buffer *in, struct buffer *out)
+buffer_poll(int fd, int events, struct buffer *in, struct buffer *out)
 {
 	ssize_t	n;
 
-#if 0
-	log_debug("buffer_poll (%ld): fd=%d, revents=%d; out=%zu in=%zu",
-	    (long) getpid(),
-	    pfd->fd, pfd->revents, BUFFER_USED(out), BUFFER_USED(in));
-#endif
-
-#ifndef BROKEN_POLL
-	if (pfd->revents & (POLLERR|POLLNVAL|POLLHUP))
+	if (events & (POLLERR|POLLNVAL))
 		return (-1);
-#endif
-	if (pfd->revents & POLLIN) {
+	if (in != NULL && events & POLLIN) {
 		buffer_ensure(in, BUFSIZ);
-		n = read(pfd->fd, BUFFER_IN(in), BUFFER_FREE(in));
-#if 0
-		log_debug("buffer_poll: fd=%d, read=%zd", pfd->fd, n);
-#endif
+		n = read(fd, BUFFER_IN(in), BUFFER_FREE(in));
 		if (n == 0)
 			return (-1);
 		if (n == -1) {
@@ -63,12 +41,10 @@ buffer_poll(struct pollfd *pfd, struct buffer *in, struct buffer *out)
 				return (-1);
 		} else
 			buffer_add(in, n);
-	}
-	if (BUFFER_USED(out) > 0 && pfd->revents & POLLOUT) {
-		n = write(pfd->fd, BUFFER_OUT(out), BUFFER_USED(out));
-#if 0
-		log_debug("buffer_poll: fd=%d, write=%zd", pfd->fd, n);
-#endif
+	} else if (events & POLLHUP)
+		return (-1);
+	if (out != NULL && BUFFER_USED(out) > 0 && events & POLLOUT) {
+		n = write(fd, BUFFER_OUT(out), BUFFER_USED(out));
 		if (n == -1) {
 			if (errno != EINTR && errno != EAGAIN)
 				return (-1);
@@ -77,23 +53,3 @@ buffer_poll(struct pollfd *pfd, struct buffer *in, struct buffer *out)
 	}
 	return (0);
 }
-
-/* Flush buffer output to socket. */
-void
-buffer_flush(int fd, struct buffer *in, struct buffer *out)
-{
-	struct pollfd	pfd;
-
-	while (BUFFER_USED(out) > 0) {
-		buffer_set(&pfd, fd, in, out);
-
-		if (poll(&pfd, 1, INFTIM) == -1) {
-			if (errno == EAGAIN || errno == EINTR)
-				continue;
-			fatal("poll failed");
-		}
-
-		if (buffer_poll(&pfd, in, out) != 0)
-			break;
-	}
-}
