@@ -1,4 +1,4 @@
-/* $Id: cfg.c,v 1.15 2009/03/31 22:23:43 nicm Exp $ */
+/* $Id: cfg.c,v 1.23 2009/10/28 23:12:38 tcunha Exp $ */
 
 /*
  * Copyright (c) 2008 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -17,6 +17,7 @@
  */
 
 #include <sys/types.h>
+#include <sys/stat.h>
 
 #include <errno.h>
 #include <stdio.h>
@@ -29,7 +30,6 @@
  * argv array and executed as a command.
  */
 
-char	 *cfg_string(FILE *, char, int);
 void printflike2 cfg_print(struct cmd_ctx *, const char *, ...);
 void printflike2 cfg_error(struct cmd_ctx *, const char *, ...);
 
@@ -51,11 +51,11 @@ cfg_error(unused struct cmd_ctx *ctx, const char *fmt, ...)
 }
 
 int
-load_cfg(const char *path, char **cause)
+load_cfg(const char *path, struct cmd_ctx *ctxin, char **cause)
 {
-	FILE   	        *f;
+	FILE		*f;
 	u_int		 n;
-	char	        *buf, *line, *ptr;
+	char		*buf, *line, *ptr;
 	size_t		 len;
 	struct cmd_list	*cmdlist;
 	struct cmd_ctx	 ctx;
@@ -87,16 +87,20 @@ load_cfg(const char *path, char **cause)
 			continue;
 		cfg_cause = NULL;
 
-		ctx.msgdata = NULL;
-		ctx.cursession = NULL;
-		ctx.curclient = NULL;
+		if (ctxin == NULL) {
+			ctx.msgdata = NULL;
+			ctx.curclient = NULL;
+			ctx.cmdclient = NULL;
+		} else {
+			ctx.msgdata = ctxin->msgdata;
+			ctx.curclient = ctxin->curclient;
+			ctx.cmdclient = ctxin->cmdclient;
+		}
 
 		ctx.error = cfg_error;
 		ctx.print = cfg_print;
 		ctx.info = cfg_print;
 
-		ctx.cmdclient = NULL;
-
 		cfg_cause = NULL;
 		cmd_list_exec(cmdlist, &ctx);
 		cmd_list_free(cmdlist);
@@ -112,6 +116,8 @@ load_cfg(const char *path, char **cause)
 	return (0);
 
 error:
+	if (line != NULL)
+		xfree(line);
 	fclose(f);
 
 	xasprintf(&ptr, "%s: %s at line %u", path, *cause, n);
