@@ -1,4 +1,4 @@
-/* $Id: clock.c,v 1.3 2009/03/27 16:59:57 nicm Exp $ */
+/* $Id: clock.c,v 1.7 2009/09/11 14:13:52 tcunha Exp $ */
 
 /*
  * Copyright (c) 2009 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -112,8 +112,6 @@ clock_draw(struct screen_write_ctx *ctx, u_int colour, int style)
 		strftime(tim, sizeof tim, "%H:%M", localtime(&t));
 
 	screen_write_clearscreen(ctx);
-	memcpy(&gc, &grid_default_cell, sizeof gc);
-	gc.fg = colour;
 
 	if (screen_size_x(s) < 6 * strlen(tim) || screen_size_y(s) < 6) {
 		if (screen_size_x(s) >= strlen(tim) && screen_size_y(s) != 0) {
@@ -121,15 +119,18 @@ clock_draw(struct screen_write_ctx *ctx, u_int colour, int style)
 			y = screen_size_y(s) / 2;
 			screen_write_cursormove(ctx, x, y);
 
-			gc.fg = colour;
+			memcpy(&gc, &grid_default_cell, sizeof gc);
+			colour_set_fg(&gc, colour);
 			screen_write_puts(ctx, &gc, "%s", tim);
 		}
 		return;
-	}		
+	}
 
 	x = (screen_size_x(s) / 2) - 3 * strlen(tim);
 	y = (screen_size_y(s) / 2) - 3;
 
+	memcpy(&gc, &grid_default_cell, sizeof gc);
+	colour_set_bg(&gc, colour);
 	for (ptr = tim; *ptr != '\0'; ptr++) {
 		if (*ptr >= '0' && *ptr <= '9')
 			idx = *ptr - '0';
@@ -147,13 +148,10 @@ clock_draw(struct screen_write_ctx *ctx, u_int colour, int style)
 		}
 
 		for (j = 0; j < 5; j++) {
-			screen_write_cursormove(ctx, x, y + j);
 			for (i = 0; i < 5; i++) {
+				screen_write_cursormove(ctx, x + i, y + j);
 				if (clock_table[idx][j][i])
-					gc.attr |= GRID_ATTR_REVERSE;
-				else
-					gc.attr &= ~GRID_ATTR_REVERSE;
-				screen_write_putc(ctx, &gc, ' ');
+					screen_write_putc(ctx, &gc, ' ');
 			}
 		}
 		x += 6;
