@@ -1,4 +1,4 @@
-/* $Id: cmd-choose-session.c,v 1.6 2009/01/19 18:23:40 nicm Exp $ */
+/* $Id: cmd-choose-session.c,v 1.14 2009/10/11 23:38:16 tcunha Exp $ */
 
 /*
  * Copyright (c) 2009 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -18,6 +18,8 @@
 
 #include <sys/types.h>
 
+#include <ctype.h>
+
 #include "tmux.h"
 
 /*
@@ -27,22 +29,22 @@
 int	cmd_choose_session_exec(struct cmd *, struct cmd_ctx *);
 
 void	cmd_choose_session_callback(void *, int);
+void	cmd_choose_session_free(void *);
 
 const struct cmd_entry cmd_choose_session_entry = {
 	"choose-session", NULL,
-	CMD_TARGET_WINDOW_USAGE,
-	0,
+	CMD_TARGET_WINDOW_USAGE " [template]",
+	CMD_ARG01, 0,
 	cmd_target_init,
 	cmd_target_parse,
 	cmd_choose_session_exec,
-	cmd_target_send,
-	cmd_target_recv,
 	cmd_target_free,
 	cmd_target_print
 };
 
 struct cmd_choose_session_data {
-	u_int	client;
+	struct client	*client;
+	char   		*template;
 };
 
 int
@@ -52,7 +54,9 @@ cmd_choose_session_exec(struct cmd *self, struct cmd_ctx *ctx)
 	struct cmd_choose_session_data	*cdata;
 	struct winlink			*wl;
 	struct session			*s;
+	struct session_group		*sg;
 	u_int			 	 i, idx, cur;
+	char				 tmp[64];
 
 	if (ctx->curclient == NULL) {
 		ctx->error(ctx, "must be run interactively");
@@ -61,7 +65,7 @@ cmd_choose_session_exec(struct cmd *self, struct cmd_ctx *ctx)
 
 	if ((wl = cmd_find_window(ctx, data->target, NULL)) == NULL)
 		return (-1);
-		
+
 	if (window_pane_set_mode(wl->window->active, &window_choose_mode) != 0)
 		return (0);
 
@@ -74,17 +78,30 @@ cmd_choose_session_exec(struct cmd *self, struct cmd_ctx *ctx)
 			cur = idx;
 		idx++;
 
+		sg = session_group_find(s);
+		if (sg == NULL)
+			*tmp = '\0';
+		else {
+			idx = session_group_index(sg);
+			xsnprintf(tmp, sizeof tmp, " (group %u)", idx);
+		}
+
 		window_choose_add(wl->window->active, i,
-		    "%s: %u windows [%ux%u]%s", s->name,
+		    "%s: %u windows [%ux%u]%s%s", s->name,
 		    winlink_count(&s->windows), s->sx, s->sy,
-		    s->flags & SESSION_UNATTACHED ? "" : " (attached)");
+		    tmp, s->flags & SESSION_UNATTACHED ? "" : " (attached)");
 	}
 
 	cdata = xmalloc(sizeof *cdata);
-	cdata->client = server_client_index(ctx->curclient);
+	if (data->arg != NULL)
+		cdata->template = xstrdup(data->arg);
+	else
+		cdata->template = xstrdup("switch-client -t '%%'");
+	cdata->client = ctx->curclient;
+	cdata->client->references++;
 
-	window_choose_ready(
-	    wl->window->active, cur, cmd_choose_session_callback, cdata);
+	window_choose_ready(wl->window->active,
+	    cur, cmd_choose_session_callback, cmd_choose_session_free, cdata);
 
 	return (0);
 }
@@ -93,15 +110,53 @@ void
 cmd_choose_session_callback(void *data, int idx)
 {
 	struct cmd_choose_session_data	*cdata = data;
-	struct client  			*c;
+	struct session			*s;
+	struct cmd_list			*cmdlist;
+	struct cmd_ctx			 ctx;
+	char				*template, *cause;
 
-	if (idx != -1 && cdata->client <= ARRAY_LENGTH(&clients) - 1) {
-		c = ARRAY_ITEM(&clients, cdata->client);
-		if (c != NULL && (u_int) idx <= ARRAY_LENGTH(&sessions) - 1) {
-			c->session = ARRAY_ITEM(&sessions, idx);
-			recalculate_sizes();
-			server_redraw_client(c);
+	if (idx == -1)
+		return;
+	if (cdata->client->flags & CLIENT_DEAD)
+		return;
+
+	if ((u_int) idx > ARRAY_LENGTH(&sessions) - 1)
+		return;
+	s = ARRAY_ITEM(&sessions, idx);
+	if (s == NULL)
+		return;
+	template = cmd_template_replace(cdata->template, s->name, 1);
+
+	if (cmd_string_parse(template, &cmdlist, &cause) != 0) {
+		if (cause != NULL) {
+			*cause = toupper((u_char) *cause);
+			status_message_set(cdata->client, "%s", cause);
+			xfree(cause);
 		}
+		xfree(template);
+		return;
 	}
+	xfree(template);
+
+	ctx.msgdata = NULL;
+	ctx.curclient = cdata->client;
+
+	ctx.error = key_bindings_error;
+	ctx.print = key_bindings_print;
+	ctx.info = key_bindings_info;
+
+	ctx.cmdclient = NULL;
+
+	cmd_list_exec(cmdlist, &ctx);
+	cmd_list_free(cmdlist);
+}
+
+void
+cmd_choose_session_free(void *data)
+{
+	struct cmd_choose_session_data	*cdata = data;
+
+	cdata->client->references--;
+	xfree(cdata->template);
 	xfree(cdata);
 }
