@@ -1,4 +1,4 @@
-/* $Id: cmd-copy-mode.c,v 1.16 2009/02/25 21:56:46 nicm Exp $ */
+/* $Id: cmd-copy-mode.c,v 1.24 2009/10/06 14:14:06 tcunha Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -24,33 +24,47 @@
  * Enter copy mode.
  */
 
+void	cmd_copy_mode_init(struct cmd *, int);
 int	cmd_copy_mode_exec(struct cmd *, struct cmd_ctx *);
 
 const struct cmd_entry cmd_copy_mode_entry = {
 	"copy-mode", NULL,
-	CMD_TARGET_WINDOW_USAGE,
-	CMD_UFLAG,
-	cmd_target_init,
+	"[-u] " CMD_TARGET_PANE_USAGE,
+	0, CMD_CHFLAG('u'),
+	cmd_copy_mode_init,
 	cmd_target_parse,
 	cmd_copy_mode_exec,
-	cmd_target_send,
-	cmd_target_recv,
 	cmd_target_free,
 	NULL
 };
 
+void
+cmd_copy_mode_init(struct cmd *self, int key)
+{
+	struct cmd_target_data	*data;
+
+	cmd_target_init(self, key);
+	data = self->data;
+
+	switch (key) {
+	case KEYC_PPAGE:
+		data->chflags |= CMD_CHFLAG('u');
+		break;
+	}
+}
+
 int
 cmd_copy_mode_exec(struct cmd *self, struct cmd_ctx *ctx)
 {
 	struct cmd_target_data	*data = self->data;
-	struct winlink		*wl;
+	struct window_pane	*wp;
 
-	if ((wl = cmd_find_window(ctx, data->target, NULL)) == NULL)
+	if (cmd_find_pane(ctx, data->target, NULL, &wp) == NULL)
 		return (-1);
 
-	window_pane_set_mode(wl->window->active, &window_copy_mode);
-	if (data->flags & CMD_UFLAG)
-		window_copy_pageup(wl->window->active);
+	window_pane_set_mode(wp, &window_copy_mode);
+	if (wp->mode == &window_copy_mode && data->chflags & CMD_CHFLAG('u'))
+		window_copy_pageup(wp);
 
 	return (0);
 }
