@@ -1,4 +1,4 @@
-/* $Id: cmd-delete-buffer.c,v 1.4 2009/01/19 18:23:40 nicm Exp $ */
+/* $Id: cmd-delete-buffer.c,v 1.7 2009/07/28 22:12:16 tcunha Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -31,12 +31,10 @@ int	cmd_delete_buffer_exec(struct cmd *, struct cmd_ctx *);
 const struct cmd_entry cmd_delete_buffer_entry = {
 	"delete-buffer", "deleteb",
 	CMD_BUFFER_SESSION_USAGE,
-	0,
+	0, 0,
 	cmd_buffer_init,
 	cmd_buffer_parse,
 	cmd_delete_buffer_exec,
-	cmd_buffer_send,
-	cmd_buffer_recv,
 	cmd_buffer_free,
 	cmd_buffer_print
 };
@@ -56,6 +54,6 @@ cmd_delete_buffer_exec(struct cmd *self, struct cmd_ctx *ctx)
 		ctx->error(ctx, "no buffer %d", data->buffer);
 		return (-1);
 	}
-	
+
 	return (0);
 }
