@@ -1,4 +1,4 @@
-/* $Id: cmd-find-window.c,v 1.6 2009/03/29 11:18:28 nicm Exp $ */
+/* $Id: cmd-find-window.c,v 1.13 2009/07/28 22:12:16 tcunha Exp $ */
 
 /*
  * Copyright (c) 2009 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -18,6 +18,7 @@
 
 #include <sys/types.h>
 
+#include <fnmatch.h>
 #include <string.h>
 
 #include "tmux.h"
@@ -29,17 +30,14 @@
 int	cmd_find_window_exec(struct cmd *, struct cmd_ctx *);
 
 void	cmd_find_window_callback(void *, int);
-char   *cmd_find_window_search(struct window_pane *, const char *);
 
 const struct cmd_entry cmd_find_window_entry = {
 	"find-window", "findw",
 	CMD_TARGET_WINDOW_USAGE " match-string",
-	CMD_ARG1,
+	CMD_ARG1, 0,
 	cmd_target_init,
 	cmd_target_parse,
 	cmd_find_window_exec,
-	cmd_target_send,
-	cmd_target_recv,
 	cmd_target_free,
 	cmd_target_print
 };
@@ -59,8 +57,8 @@ cmd_find_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 	struct window_pane		*wp;
 	ARRAY_DECL(, u_int)	 	 list_idx;
 	ARRAY_DECL(, char *)	 	 list_ctx;
-	char				*sres, *sctx;
-	u_int				 i;
+	char				*sres, *sctx, *searchstr;
+	u_int				 i, line;
 
 	if (ctx->curclient == NULL) {
 		ctx->error(ctx, "must be run interactively");
@@ -70,37 +68,41 @@ cmd_find_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 
 	if ((wl = cmd_find_window(ctx, data->target, NULL)) == NULL)
 		return (-1);
-	
+
 	ARRAY_INIT(&list_idx);
 	ARRAY_INIT(&list_ctx);
 
+	xasprintf(&searchstr, "*%s*", data->arg);
 	RB_FOREACH(wm, winlinks, &s->windows) {
 		i = 0;
 		TAILQ_FOREACH(wp, &wm->window->panes, entry) {
 			i++;
 
-			if (strstr(wm->window->name, data->arg) != NULL)
+			if (fnmatch(searchstr, wm->window->name, 0) == 0)
 				sctx = xstrdup("");
 			else {
-				sres = cmd_find_window_search(wp, data->arg);
+				sres = window_pane_search(wp, data->arg, &line);
 				if (sres == NULL &&
-				    strstr(wp->base.title, data->arg) == NULL)
+				    fnmatch(searchstr, wp->base.title, 0) != 0)
 					continue;
-				
+
 				if (sres == NULL) {
 					xasprintf(&sctx,
 					    "pane %u title: \"%s\"", i - 1,
 					    wp->base.title);
 				} else {
-					xasprintf(&sctx, "\"%s\"", sres);
+					xasprintf(&sctx,
+					    "pane %u line %u: \"%s\"", i - 1,
+					    line + 1, sres);
 					xfree(sres);
 				}
 			}
-				
+
 			ARRAY_ADD(&list_idx, wm->idx);
-			ARRAY_ADD(&list_ctx, sctx);			
+			ARRAY_ADD(&list_ctx, sctx);
 		}
 	}
+	xfree(searchstr);
 
 	if (ARRAY_LENGTH(&list_idx) == 0) {
 		ctx->error(ctx, "no windows matching: %s", data->arg);
@@ -123,7 +125,7 @@ cmd_find_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 		wm = winlink_find_by_index(
 		    &s->windows, ARRAY_ITEM(&list_idx, i));
 		w = wm->window;
-		
+
 		sctx = ARRAY_ITEM(&list_ctx, i);
 		window_choose_add(wl->window->active,
 		    wm->idx, "%3d: %s [%ux%u] (%u panes) %s", wm->idx, w->name,
@@ -136,7 +138,7 @@ cmd_find_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 		fatalx("session not found");
 
 	window_choose_ready(
-	    wl->window->active, 0, cmd_find_window_callback, cdata);
+	    wl->window->active, 0, cmd_find_window_callback, xfree, cdata);
 
 out:
 	ARRAY_FREE(&list_idx);
@@ -157,48 +159,4 @@ cmd_find_window_callback(void *data, int idx)
 			server_redraw_session(s);
 		recalculate_sizes();
 	}
-	xfree(cdata);
-}
-
-char *
-cmd_find_window_search(struct window_pane *wp, const char *searchstr)
-{
-	const struct grid_cell	*gc;
-	const struct grid_utf8	*gu;
-	char			*buf, *s;
-	size_t	 		 off;
-	u_int	 		 i, j, k;
-
-	buf = xmalloc(1);
-				
-	for (j = 0; j < screen_size_y(&wp->base); j++) {
-		off = 0;
-		for (i = 0; i < screen_size_x(&wp->base); i++) {
-			gc = grid_view_peek_cell(wp->base.grid, i, j);
-			if (gc->flags & GRID_FLAG_UTF8) {
-				gu = grid_view_peek_utf8(wp->base.grid, i, j);
-				buf = xrealloc(buf, 1, off + 8);
-				for (k = 0; k < UTF8_SIZE; k++) {
-					if (gu->data[k] == 0xff)
-						break;
-					buf[off++] = gu->data[k];
-				}
-			} else {
-				buf = xrealloc(buf, 1, off + 1);
-				buf[off++] = gc->data;
-			}
-		}
-		while (off > 0 && buf[off - 1] == ' ')
-			off--;
-		buf[off] = '\0';
-		
-		if ((s = strstr(buf, searchstr)) != NULL) {
-			s = section_string(buf, off, s - buf, 40);
-			xfree(buf);
-			return (s);
-		}
-	}
-
-	xfree(buf);
-	return (NULL);
 }
