@@ -1,4 +1,4 @@
-/* $Id: cmd-kill-server.c,v 1.6 2009/01/19 18:23:40 nicm Exp $ */
+/* $Id: cmd-kill-server.c,v 1.8 2009/07/28 22:12:16 tcunha Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -32,13 +32,11 @@ int	cmd_kill_server_exec(struct cmd *, struct cmd_ctx *);
 const struct cmd_entry cmd_kill_server_entry = {
 	"kill-server", NULL,
 	"",
-	0,
+	0, 0,
 	NULL,
 	NULL,
 	cmd_kill_server_exec,
 	NULL,
-	NULL,
-	NULL,
 	NULL
 };
 
