@@ -1,4 +1,4 @@
-/* $Id: cmd-list-buffers.c,v 1.7 2009/01/19 18:23:40 nicm Exp $ */
+/* $Id: cmd-list-buffers.c,v 1.12 2009/09/07 23:48:54 tcunha Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -31,12 +31,10 @@ int	cmd_list_buffers_exec(struct cmd *, struct cmd_ctx *);
 const struct cmd_entry cmd_list_buffers_entry = {
 	"list-buffers", "lsb",
 	CMD_TARGET_SESSION_USAGE,
-	0,
+	0, 0,
 	cmd_target_init,
 	cmd_target_parse,
 	cmd_list_buffers_exec,
-	cmd_target_send,
-	cmd_target_recv,
 	cmd_target_free,
 	cmd_target_print
 };
@@ -48,44 +46,33 @@ cmd_list_buffers_exec(struct cmd *self, struct cmd_ctx *ctx)
 	struct session		*s;
 	struct paste_buffer	*pb;
 	u_int			 idx;
-	char			*tmp;
-	size_t			 size, in, out;
+	char			 tmp[51 * 4 + 1];
+	size_t			 size, len;
 
 	if ((s = cmd_find_session(ctx, data->target)) == NULL)
 		return (-1);
 
-	if (s->sx > 35) {	/* leave three for ... */
-		size = s->sx - 32;
-		tmp = xmalloc(size + 1);
-	} else {
-		size = 0;
-		tmp = NULL;
-	}
-
 	idx = 0;
 	while ((pb = paste_walk_stack(&s->buffers, &idx)) != NULL) {
-		if (tmp != NULL) {
-			in = out = 0;
-			while (out < size && pb->data[in] != '\0') {
-				if (pb->data[in] > 31 && pb->data[in] != 127)
-					tmp[out++] = pb->data[in];
-				in++;
-			}
-			tmp[out] = '\0';
-			if (out == size) {
-				tmp[out - 1] = '.';
-				tmp[out - 2] = '.';
-				tmp[out - 3] = '.';
-			}
+		size = pb->size;
 
-			ctx->print(ctx, "%d: %zu bytes: \"%s\"",
-			    idx - 1, strlen(pb->data), tmp);
-		} else
-			ctx->print(ctx, "%d: %zu bytes", idx, strlen(pb->data));
+		/* Translate the first 50 characters. */
+		len = size;
+		if (len > 50)
+			len = 50;
+		strvisx(tmp, pb->data, len, VIS_OCTAL|VIS_TAB|VIS_NL);
+
+		/*
+		 * If the first 50 characterswere encoded as a longer string,
+		 * or there is definitely more data, add "...".
+		 */
+		if (size > 50 || strlen(tmp) > 50) {
+			tmp[50 - 3] = '\0';
+			strlcat(tmp, "...", sizeof tmp);
+		}
+		
+		ctx->print(ctx, "%u: %zu bytes: \"%s\"", idx - 1, size, tmp);
 	}
 
-	if (tmp != NULL)
-		xfree(tmp);
-
 	return (0);
 }
