@@ -1,4 +1,4 @@
-/* $Id: cmd-list-commands.c,v 1.3 2009/01/19 18:23:40 nicm Exp $ */
+/* $Id: cmd-list-commands.c,v 1.5 2009/07/28 22:12:16 tcunha Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -29,13 +29,11 @@ int	cmd_list_commands_exec(struct cmd *, struct cmd_ctx *);
 const struct cmd_entry cmd_list_commands_entry = {
 	"list-commands", "lscm",
 	"",
-	0,
+	0, 0,
 	NULL,
 	NULL,
 	cmd_list_commands_exec,
 	NULL,
-	NULL,
-	NULL,
 	NULL
 };
 
