@@ -1,4 +1,4 @@
-/* $Id: cmd-list.c,v 1.3 2009/02/16 18:57:16 nicm Exp $ */
+/* $Id: cmd-list.c,v 1.6 2009/07/28 22:12:16 tcunha Exp $ */
 
 /*
  * Copyright (c) 2009 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -28,29 +28,44 @@ cmd_list_parse(int argc, char **argv, char **cause)
 	struct cmd_list	*cmdlist;
 	struct cmd	*cmd;
 	int		 i, lastsplit;
+	size_t		 arglen, new_argc;
+	char	       **new_argv;
 
 	cmdlist = xmalloc(sizeof *cmdlist);
 	TAILQ_INIT(cmdlist);
 
 	lastsplit = 0;
 	for (i = 0; i < argc; i++) {
-		if (strcmp(argv[i], "\\;") == 0) {
-			argv[i][0] = ';';
-			argv[i][1] = '\0';
-		} else if (strcmp(argv[i], ";") == 0) {
-			cmd = cmd_parse(i - lastsplit, argv + lastsplit, cause);
-			if (cmd == NULL)
-				goto bad;
-			TAILQ_INSERT_TAIL(cmdlist, cmd, qentry);
-			lastsplit = i + 1;
+		arglen = strlen(argv[i]);
+		if (arglen == 0 || argv[i][arglen - 1] != ';')
+			continue;
+		argv[i][arglen - 1] = '\0';
+
+		if (arglen > 1 && argv[i][arglen - 2] == '\\') {
+			argv[i][arglen - 2] = ';';
+			continue;
 		}
+
+		new_argc = i - lastsplit;
+		new_argv = argv + lastsplit;
+		if (arglen != 1)
+			new_argc++;
+
+		cmd = cmd_parse(new_argc, new_argv, cause);
+		if (cmd == NULL)
+			goto bad;
+		TAILQ_INSERT_TAIL(cmdlist, cmd, qentry);
+
+		lastsplit = i + 1;
+	}
+
+	if (lastsplit != argc) {
+		cmd = cmd_parse(argc - lastsplit, argv + lastsplit, cause);
+		if (cmd == NULL)
+			goto bad;
+		TAILQ_INSERT_TAIL(cmdlist, cmd, qentry);
 	}
 
-	cmd = cmd_parse(argc - lastsplit, argv + lastsplit, cause);
-	if (cmd == NULL)
-		goto bad;
-	TAILQ_INSERT_TAIL(cmdlist, cmd, qentry);
-       
 	return (cmdlist);
 
 bad:
@@ -71,46 +86,11 @@ cmd_list_exec(struct cmd_list *cmdlist, struct cmd_ctx *ctx)
 	return (0);
 }
 
-void
-cmd_list_send(struct cmd_list *cmdlist, struct buffer *b)
-{
-	struct cmd	*cmd;
-	u_int		 n;
-
-	n = 0;
-	TAILQ_FOREACH(cmd, cmdlist, qentry)
-	    	n++;
-
-	buffer_write(b, &n, sizeof n);
-	TAILQ_FOREACH(cmd, cmdlist, qentry)
-	    	cmd_send(cmd, b);
-}
-
-struct cmd_list *
-cmd_list_recv(struct buffer *b)
-{
-	struct cmd_list	*cmdlist;
-	struct cmd	*cmd;
-	u_int		 n;
-
-	buffer_read(b, &n, sizeof n);
-
-	cmdlist = xmalloc(sizeof *cmdlist);
-	TAILQ_INIT(cmdlist);
-
-	while (n-- > 0) {
-		cmd = cmd_recv(b);
-		TAILQ_INSERT_TAIL(cmdlist, cmd, qentry);
-	}
-
-	return (cmdlist);
-}
-
 void
 cmd_list_free(struct cmd_list *cmdlist)
 {
 	struct cmd	*cmd;
-	
+
 	while (!TAILQ_EMPTY(cmdlist)) {
 		cmd = TAILQ_FIRST(cmdlist);
 		TAILQ_REMOVE(cmdlist, cmd, qentry);
