@@ -1,4 +1,4 @@
-/* $Id: cmd-lock-server.c,v 1.2 2009/01/19 18:23:40 nicm Exp $ */
+/* $Id: cmd-lock-server.c,v 1.7 2009/09/25 17:47:42 tcunha Exp $ */
 
 /*
  * Copyright (c) 2008 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -30,25 +30,22 @@
 
 int	cmd_lock_server_exec(struct cmd *, struct cmd_ctx *);
 
-int	cmd_lock_server_callback(void *, const char *);
-
 const struct cmd_entry cmd_lock_server_entry = {
 	"lock-server", "lock",
 	"",
-	0,
+	0, 0,
 	NULL,
 	NULL,
 	cmd_lock_server_exec,
 	NULL,
 	NULL,
-	NULL,
-	NULL,
 };
 
 int
 cmd_lock_server_exec(unused struct cmd *self, unused struct cmd_ctx *ctx)
 {
 	server_lock();
+	recalculate_sizes();
 
 	return (0);
 }
