@@ -1,4 +1,4 @@
-/* $Id: cmd-next-window.c,v 1.16 2009/03/28 14:08:09 nicm Exp $ */
+/* $Id: cmd-next-window.c,v 1.20 2009/07/28 22:12:16 tcunha Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -29,13 +29,11 @@ int	cmd_next_window_exec(struct cmd *, struct cmd_ctx *);
 
 const struct cmd_entry cmd_next_window_entry = {
 	"next-window", "next",
-	CMD_TARGET_SESSION_USAGE,
-	CMD_AFLAG,
+	"[-a] " CMD_TARGET_SESSION_USAGE,
+	0, CMD_CHFLAG('a'),
 	cmd_next_window_init,
 	cmd_target_parse,
 	cmd_next_window_exec,
-	cmd_target_send,
-	cmd_target_recv,
 	cmd_target_free,
 	cmd_target_print
 };
@@ -48,8 +46,8 @@ cmd_next_window_init(struct cmd *self, int key)
 	cmd_target_init(self, key);
 	data = self->data;
 
-	if (key == KEYC_ADDESC('n'))
-		data->flags |= CMD_AFLAG;
+	if (key == ('n' | KEYC_ESCAPE))
+		data->chflags |= CMD_CHFLAG('a');
 }
 
 int
@@ -63,7 +61,7 @@ cmd_next_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 		return (-1);
 
 	activity = 0;
-	if (data->flags & CMD_AFLAG)
+	if (data->chflags & CMD_CHFLAG('a'))
 		activity = 1;
 
 	if (session_next(s, activity) == 0)
