@@ -1,4 +1,4 @@
-/* $Id: cmd-rename-session.c,v 1.15 2009/01/19 18:23:40 nicm Exp $ */
+/* $Id: cmd-rename-session.c,v 1.18 2009/07/28 22:12:16 tcunha Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -31,12 +31,10 @@ int	cmd_rename_session_exec(struct cmd *, struct cmd_ctx *);
 const struct cmd_entry cmd_rename_session_entry = {
 	"rename-session", "rename",
 	CMD_TARGET_SESSION_USAGE " new-name",
-	CMD_ARG1,
+	CMD_ARG1, 0,
 	cmd_target_init,
 	cmd_target_parse,
 	cmd_rename_session_exec,
-	cmd_target_send,
-	cmd_target_recv,
 	cmd_target_free,
 	cmd_target_print
 };
@@ -53,5 +51,7 @@ cmd_rename_session_exec(struct cmd *self, struct cmd_ctx *ctx)
 	xfree(s->name);
 	s->name = xstrdup(data->arg);
 
+	server_status_session(s);
+
 	return (0);
 }
