@@ -1,4 +1,4 @@
-/* $Id: cmd-rename-window.c,v 1.26 2009/01/20 19:35:03 nicm Exp $ */
+/* $Id: cmd-rename-window.c,v 1.30 2009/10/11 23:38:16 tcunha Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -31,12 +31,10 @@ int	cmd_rename_window_exec(struct cmd *, struct cmd_ctx *);
 const struct cmd_entry cmd_rename_window_entry = {
 	"rename-window", "renamew",
 	CMD_TARGET_WINDOW_USAGE " new-name",
-	CMD_ARG1,
+	CMD_ARG1, 0,
 	cmd_target_init,
 	cmd_target_parse,
 	cmd_rename_window_exec,
-	cmd_target_send,
-	cmd_target_recv,
 	cmd_target_free,
 	cmd_target_print
 };
@@ -53,9 +51,9 @@ cmd_rename_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 
 	xfree(wl->window->name);
 	wl->window->name = xstrdup(data->arg);
-	options_set_number(&wl->window->options, "automatic-rename", 0);	
+	options_set_number(&wl->window->options, "automatic-rename", 0);
 
-	server_status_session(s);
+	server_status_window(wl->window);
 
 	return (0);
 }
