@@ -1,4 +1,4 @@
-/* $Id: cmd-select-prompt.c,v 1.7 2009/02/13 18:57:55 nicm Exp $ */
+/* $Id: cmd-select-prompt.c,v 1.12 2009/08/16 19:29:24 tcunha Exp $ */
 
 /*
  * Copyright (c) 2008 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -33,12 +33,10 @@ int	cmd_select_prompt_callback(void *, const char *);
 const struct cmd_entry cmd_select_prompt_entry = {
 	"select-prompt", NULL,
 	CMD_TARGET_CLIENT_USAGE,
-	0,
+	0, 0,
 	cmd_target_init,
 	cmd_target_parse,
 	cmd_select_prompt_exec,
-	cmd_target_send,
-	cmd_target_recv,
 	cmd_target_free,
 	cmd_target_print
 };
@@ -55,7 +53,7 @@ cmd_select_prompt_exec(struct cmd *self, struct cmd_ctx *ctx)
 	if (c->prompt_string != NULL)
 		return (0);
 
-	status_prompt_set(c, "index ", cmd_select_prompt_callback, c, 0);
+	status_prompt_set(c, "index ", cmd_select_prompt_callback, NULL, c, 0);
 
 	return (0);
 }
@@ -68,20 +66,20 @@ cmd_select_prompt_callback(void *data, const char *s)
 	char		 msg[128];
 	u_int		 idx;
 
-	if (s == NULL)
+	if (s == NULL || *s == '\0')
 		return (0);
 
 	idx = strtonum(s, 0, UINT_MAX, &errstr);
 	if (errstr != NULL) {
 		xsnprintf(msg, sizeof msg, "Index %s: %s", errstr, s);
-		status_message_set(c, msg);
+		status_message_set(c, "%s", msg);
 		return (0);
 	}
 
 	if (winlink_find_by_index(&c->session->windows, idx) == NULL) {
 		xsnprintf(msg, sizeof msg,
 		    "Window not found: %s:%d", c->session->name, idx);
-		status_message_set(c, msg);
+		status_message_set(c, "%s", msg);
 		return (0);
 	}
 
