@@ -1,4 +1,4 @@
-/* $Id: cmd-select-window.c,v 1.21 2009/01/19 18:23:40 nicm Exp $ */
+/* $Id: cmd-select-window.c,v 1.23 2009/07/28 22:12:16 tcunha Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -32,12 +32,10 @@ int	cmd_select_window_exec(struct cmd *, struct cmd_ctx *);
 const struct cmd_entry cmd_select_window_entry = {
 	"select-window", "selectw",
 	CMD_TARGET_WINDOW_USAGE,
-	0,
+	0, 0,
 	cmd_select_window_init,
 	cmd_target_parse,
 	cmd_select_window_exec,
-	cmd_target_send,
-	cmd_target_recv,
 	cmd_target_free,
 	cmd_target_print
 };
