@@ -1,4 +1,4 @@
-/* $Id: cmd-set-buffer.c,v 1.7 2009/01/25 18:51:28 tcunha Exp $ */
+/* $Id: cmd-set-buffer.c,v 1.10 2009/09/07 23:48:54 tcunha Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -18,7 +18,7 @@
 
 #include <sys/types.h>
 
-#include <stdlib.h>
+#include <string.h>
 
 #include "tmux.h"
 
@@ -31,12 +31,10 @@ int	cmd_set_buffer_exec(struct cmd *, struct cmd_ctx *);
 const struct cmd_entry cmd_set_buffer_entry = {
 	"set-buffer", "setb",
 	CMD_BUFFER_SESSION_USAGE " data",
-	CMD_ARG1,
+	CMD_ARG1, 0,
 	cmd_buffer_init,
 	cmd_buffer_parse,
 	cmd_set_buffer_exec,
-	cmd_buffer_send,
-	cmd_buffer_recv,
 	cmd_buffer_free,
 	cmd_buffer_print
 };
@@ -47,17 +45,23 @@ cmd_set_buffer_exec(struct cmd *self, struct cmd_ctx *ctx)
 	struct cmd_buffer_data	*data = self->data;
 	struct session		*s;
 	u_int			 limit;
+	u_char			*pdata;
+	size_t			 psize;
 
 	if ((s = cmd_find_session(ctx, data->target)) == NULL)
 		return (-1);
-
 	limit = options_get_number(&s->options, "buffer-limit");
+
+	pdata = xstrdup(data->arg);
+	psize = strlen(pdata);
+
 	if (data->buffer == -1) {
-		paste_add(&s->buffers, xstrdup(data->arg), limit);
+		paste_add(&s->buffers, pdata, psize, limit);
 		return (0);
 	}
-	if (paste_replace(&s->buffers, data->buffer, xstrdup(data->arg)) != 0) {
+	if (paste_replace(&s->buffers, data->buffer, pdata, psize) != 0) {
 		ctx->error(ctx, "no buffer %d", data->buffer);
+		xfree(pdata);
 		return (-1);
 	}
 	return (0);
