@@ -1,4 +1,4 @@
-/* $Id: cmd-start-server.c,v 1.6 2009/01/19 18:23:40 nicm Exp $ */
+/* $Id: cmd-start-server.c,v 1.8 2009/07/28 22:12:16 tcunha Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -29,13 +29,11 @@ int	cmd_start_server_exec(struct cmd *, struct cmd_ctx *);
 const struct cmd_entry cmd_start_server_entry = {
 	"start-server", "start",
 	"",
-	CMD_STARTSERVER,
+	CMD_STARTSERVER, 0,
 	NULL,
 	NULL,
 	cmd_start_server_exec,
 	NULL,
-	NULL,
-	NULL,
 	NULL
 };
 
