@@ -1,4 +1,4 @@
-/* $Id: input.c,v 1.76 2009/03/28 20:17:29 nicm Exp $ */
+/* $Id: input.c,v 1.101 2009/10/28 23:12:38 tcunha Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -35,7 +35,7 @@
 #define INPUT_SPECIAL(ch)	(ch == 0xff)
 
 int	 input_get_argument(struct input_ctx *, u_int, uint16_t *, uint16_t);
-int	 input_new_argument(struct input_ctx *);
+void	 input_new_argument(struct input_ctx *);
 int	 input_add_argument(struct input_ctx *, u_char);
 
 void	 input_start_string(struct input_ctx *, int);
@@ -67,6 +67,8 @@ void	 input_handle_sequence_cud(struct input_ctx *);
 void	 input_handle_sequence_cuf(struct input_ctx *);
 void	 input_handle_sequence_cub(struct input_ctx *);
 void	 input_handle_sequence_dch(struct input_ctx *);
+void	 input_handle_sequence_cbt(struct input_ctx *);
+void	 input_handle_sequence_da(struct input_ctx *);
 void	 input_handle_sequence_dl(struct input_ctx *);
 void	 input_handle_sequence_ich(struct input_ctx *);
 void	 input_handle_sequence_il(struct input_ctx *);
@@ -74,6 +76,7 @@ void	 input_handle_sequence_vpa(struct input_ctx *);
 void	 input_handle_sequence_hpa(struct input_ctx *);
 void	 input_handle_sequence_cup(struct input_ctx *);
 void	 input_handle_sequence_cup(struct input_ctx *);
+void	 input_handle_sequence_tbc(struct input_ctx *);
 void	 input_handle_sequence_ed(struct input_ctx *);
 void	 input_handle_sequence_el(struct input_ctx *);
 void	 input_handle_sequence_sm(struct input_ctx *);
@@ -101,8 +104,11 @@ const struct input_sequence_entry input_sequence_table[] = {
 	{ 'L', input_handle_sequence_il },
 	{ 'M', input_handle_sequence_dl },
 	{ 'P', input_handle_sequence_dch },
+	{ 'Z', input_handle_sequence_cbt },
+	{ 'c', input_handle_sequence_da },
 	{ 'd', input_handle_sequence_vpa },
 	{ 'f', input_handle_sequence_cup },
+	{ 'g', input_handle_sequence_tbc },
 	{ 'h', input_handle_sequence_sm },
 	{ 'l', input_handle_sequence_rm },
 	{ 'm', input_handle_sequence_sgr },
@@ -119,23 +125,21 @@ input_sequence_cmp(const void *a, const void *b)
 	return (ai - bi);
 }
 
-int
+void
 input_new_argument(struct input_ctx *ictx)
 {
-	struct input_arg       *arg;
+	struct input_arg	*arg;
 
 	ARRAY_EXPAND(&ictx->args, 1);
 
 	arg = &ARRAY_LAST(&ictx->args);
 	arg->used = 0;
-
-	return (0);
 }
 
 int
 input_add_argument(struct input_ctx *ictx, u_char ch)
 {
-	struct input_arg       *arg;
+	struct input_arg	*arg;
 
 	if (ARRAY_LENGTH(&ictx->args) == 0)
 		return (0);
@@ -233,6 +237,8 @@ input_init(struct window_pane *wp)
 	ictx->saved_cy = 0;
 
 	input_state(ictx, input_state_first);
+
+	ictx->was = 0;
 }
 
 void
@@ -250,8 +256,9 @@ input_parse(struct window_pane *wp)
 	struct input_ctx	*ictx = &wp->ictx;
 	u_char			 ch;
 
-	if (BUFFER_USED(wp->in) == 0)
+	if (BUFFER_USED(wp->in) == ictx->was)
 		return;
+	wp->window->flags |= WINDOW_ACTIVITY;
 
 	ictx->buf = BUFFER_OUT(wp->in);
 	ictx->len = BUFFER_USED(wp->in);
@@ -259,15 +266,11 @@ input_parse(struct window_pane *wp)
 
 	ictx->wp = wp;
 
-	log_debug2("entry; buffer=%zu", ictx->len);
-
 	if (wp->mode == NULL)
 		screen_write_start(&ictx->ctx, wp, &wp->base);
 	else
 		screen_write_start(&ictx->ctx, NULL, &wp->base);
 
-	if (ictx->off != ictx->len)
-		wp->window->flags |= WINDOW_ACTIVITY;
 	while (ictx->off < ictx->len) {
 		ch = ictx->buf[ictx->off++];
 		ictx->state(ch, ictx);
@@ -276,6 +279,7 @@ input_parse(struct window_pane *wp)
 	screen_write_stop(&ictx->ctx);
 
 	buffer_remove(wp->in, ictx->len);
+	ictx->was = BUFFER_USED(wp->in);
 }
 
 void
@@ -394,15 +398,34 @@ input_state_sequence_first(u_char ch, struct input_ctx *ictx)
 	ictx->private = '\0';
 	ARRAY_CLEAR(&ictx->args);
 
-	input_state(ictx, input_state_sequence_next);
-
-	if (INPUT_PARAMETER(ch)) {
-		input_new_argument(ictx);
-		if (ch >= 0x3c && ch <= 0x3f) {
-			/* Private control sequence. */
-			ictx->private = ch;
+	/* Most C0 control are accepted within CSI. */
+	if (INPUT_C0CONTROL(ch)) {
+		if (ch == 0x1b) {			/* ESC */
+			/* Abort sequence and begin with new. */
+			input_state(ictx, input_state_escape);
+			return;
+		} else if (ch == 0x18 || ch == 0x1a) {	/* CAN and SUB */
+			/* Abort sequence. */
+			input_state(ictx, input_state_first);
 			return;
 		}
+
+		/* Handle C0 immediately. */
+		input_handle_c0_control(ch, ictx);
+
+		/*
+		 * Just come back to this state, in case the next character
+		 * is the start of a private sequence.
+		 */
+		return;
+	}
+
+	input_state(ictx, input_state_sequence_next);
+
+	/* Private sequence: always the first character. */
+	if (ch >= 0x3c && ch <= 0x3f) {
+		ictx->private = ch;
+		return;
 	}
 
 	/* Pass character on directly. */
@@ -423,6 +446,9 @@ input_state_sequence_next(u_char ch, struct input_ctx *ictx)
 	}
 
 	if (INPUT_PARAMETER(ch)) {
+		if (ARRAY_EMPTY(&ictx->args))
+			input_new_argument(ictx);
+
 		if (ch == ';') {
 			if (input_add_argument(ictx, '\0') != 0)
 				input_state(ictx, input_state_first);
@@ -443,6 +469,24 @@ input_state_sequence_next(u_char ch, struct input_ctx *ictx)
 		return;
 	}
 
+	/* Most C0 control are accepted within CSI. */
+	if (INPUT_C0CONTROL(ch)) {
+		if (ch == 0x1b) {			/* ESC */
+			/* Abort sequence and begin with new. */
+			input_state(ictx, input_state_escape);
+			return;
+		} else if (ch == 0x18 || ch == 0x1a) {	/* CAN and SUB */
+			/* Abort sequence. */
+			input_state(ictx, input_state_first);
+			return;
+		}
+
+		/* Handle C0 immediately. */
+		input_handle_c0_control(ch, ictx);
+
+		return;
+	}
+
 	input_state(ictx, input_state_first);
 }
 
@@ -475,7 +519,7 @@ input_state_string_next(u_char ch, struct input_ctx *ictx)
 		return;
 	}
 
-	if (ch >= 0x20 && ch != 0x7f) {
+	if (ch >= 0x20) {
 		if (input_add_string(ictx, ch) != 0)
 			input_state(ictx, input_state_first);
 		return;
@@ -528,15 +572,14 @@ input_state_string_escape(u_char ch, struct input_ctx *ictx)
 void
 input_state_utf8(u_char ch, struct input_ctx *ictx)
 {
-	log_debug2("-- un %zu: %hhu (%c)", ictx->off, ch, ch);
+	log_debug2("-- utf8 next: %zu: %hhu (%c)", ictx->off, ch, ch);
 
-	ictx->utf8_buf[ictx->utf8_off++] = ch;
-	if (--ictx->utf8_len != 0)
-		return;
+	if (utf8_append(&ictx->utf8data, ch))
+		return;		/* more to come */
 	input_state(ictx, input_state_first);
 
 	ictx->cell.flags |= GRID_FLAG_UTF8;
-	screen_write_cell(&ictx->ctx, &ictx->cell, ictx->utf8_buf);
+	screen_write_cell(&ictx->ctx, &ictx->cell, &ictx->utf8data);
 	ictx->cell.flags &= ~GRID_FLAG_UTF8;
 }
 
@@ -544,42 +587,19 @@ void
 input_handle_character(u_char ch, struct input_ctx *ictx)
 {
 	struct window_pane	*wp = ictx->wp;
-	
-	if (ch > 0x7f && options_get_number(&wp->window->options, "utf8")) {
-		/*
-		 * UTF-8 sequence.
-		 *
-		 * 11000010-11011111 C2-DF start of 2-byte sequence
-		 * 11100000-11101111 E0-EF start of 3-byte sequence
-		 * 11110000-11110100 F0-F4 start of 4-byte sequence
-		 */
-		memset(ictx->utf8_buf, 0xff, sizeof ictx->utf8_buf);
-		ictx->utf8_buf[0] = ch;
-		ictx->utf8_off = 1;
 
-		if (ch >= 0xc2 && ch <= 0xdf) {
-			log_debug2("-- u2 %zu: %hhu (%c)", ictx->off, ch, ch);
+	if (ch > 0x7f && options_get_number(&wp->window->options, "utf8")) {
+		if (utf8_open(&ictx->utf8data, ch)) {
+			log_debug2("-- utf8 size %zu: %zu: %hhu (%c)", 
+			    ictx->utf8data.size, ictx->off, ch, ch);
 			input_state(ictx, input_state_utf8);
-			ictx->utf8_len = 1;
-			return;
-		}
-		if (ch >= 0xe0 && ch <= 0xef) {
-			log_debug2("-- u3 %zu: %hhu (%c)", ictx->off, ch, ch);
-			input_state(ictx, input_state_utf8);
-			ictx->utf8_len = 2;
-			return;
-		}
-		if (ch >= 0xf0 && ch <= 0xf4) {
-			log_debug2("-- u4 %zu: %hhu (%c)", ictx->off, ch, ch);
-			input_state(ictx, input_state_utf8);
-			ictx->utf8_len = 3;
 			return;
 		}
 	}
 	log_debug2("-- ch %zu: %hhu (%c)", ictx->off, ch, ch);
 
 	ictx->cell.data = ch;
-	screen_write_cell(&ictx->ctx, &ictx->cell, ictx->utf8_buf);
+	screen_write_cell(&ictx->ctx, &ictx->cell, NULL);
 }
 
 void
@@ -593,7 +613,7 @@ input_handle_c0_control(u_char ch, struct input_ctx *ictx)
 	case '\0':	/* NUL */
 		break;
 	case '\n':	/* LF */
-		screen_write_linefeed(&ictx->ctx);
+		screen_write_linefeed(&ictx->ctx, 0);
 		break;
 	case '\r':	/* CR */
 		screen_write_carriagereturn(&ictx->ctx);
@@ -602,15 +622,22 @@ input_handle_c0_control(u_char ch, struct input_ctx *ictx)
 		ictx->wp->window->flags |= WINDOW_BELL;
 		break;
 	case '\010': 	/* BS */
-		screen_write_cursorleft(&ictx->ctx, 1);
+		screen_write_backspace(&ictx->ctx);
 		break;
 	case '\011': 	/* TAB */
-		s->cx = ((s->cx / 8) * 8) + 8;
-		if (s->cx > screen_size_x(s) - 1) {
-			s->cx = 0;
-			screen_write_cursordown(&ictx->ctx, 1);
-		}
-		screen_write_cursormove(&ictx->ctx, s->cx, s->cy);
+		/* Don't tab beyond the end of the line. */
+		if (s->cx >= screen_size_x(s) - 1)
+			break;
+
+		/* Find the next tab point, or use the last column if none. */
+		do {
+			s->cx++;
+			if (bit_test(s->tabs, s->cx))
+				break;
+		} while (s->cx < screen_size_x(s) - 1);
+		break;
+	case '\013':	/* VT */
+		screen_write_linefeed(&ictx->ctx, 0);
 		break;
 	case '\016':	/* SO */
 		ictx->cell.attr |= GRID_ATTR_CHARSET;
@@ -627,9 +654,22 @@ input_handle_c0_control(u_char ch, struct input_ctx *ictx)
 void
 input_handle_c1_control(u_char ch, struct input_ctx *ictx)
 {
+	struct screen  *s = ictx->ctx.s;
+
 	log_debug2("-- c1 %zu: %hhu (%c)", ictx->off, ch, ch);
 
 	switch (ch) {
+	case 'D':	/* IND */
+		screen_write_linefeed(&ictx->ctx, 0);
+		break;
+	case 'E': 	/* NEL */
+		screen_write_carriagereturn(&ictx->ctx);
+		screen_write_linefeed(&ictx->ctx, 0);
+		break;
+	case 'H':	/* HTS */
+		if (s->cx < screen_size_x(s))
+			bit_set(s->tabs, s->cx);
+		break;
 	case 'M':	/* RI */
 		screen_write_reverseindex(&ictx->ctx);
 		break;
@@ -648,7 +688,7 @@ input_handle_private_two(u_char ch, struct input_ctx *ictx)
 	    "-- p2 %zu: %hhu (%c) %hhu", ictx->off, ch, ch, ictx->intermediate);
 
 	switch (ch) {
-	case '0':	/* Dscs (graphics) */
+	case '0':	/* SCS */
 		/*
 		 * Not really supported, but fake it up enough for those that
 		 * use it to switch character sets (by redefining G0 to
@@ -661,22 +701,36 @@ input_handle_private_two(u_char ch, struct input_ctx *ictx)
 		}
 		break;
 	case '=':	/* DECKPAM */
+		if (ictx->intermediate != '\0')
+			break;
 		screen_write_kkeypadmode(&ictx->ctx, 1);
 		log_debug("kkeypad on (application mode)");
 		break;
 	case '>':	/* DECKPNM */
+		if (ictx->intermediate != '\0')
+			break;
 		screen_write_kkeypadmode(&ictx->ctx, 0);
 		log_debug("kkeypad off (number mode)");
 		break;
 	case '7':	/* DECSC */
+		if (ictx->intermediate != '\0')
+			break;
 		memcpy(&ictx->saved_cell, &ictx->cell, sizeof ictx->saved_cell);
 		ictx->saved_cx = s->cx;
 		ictx->saved_cy = s->cy;
 		break;
-	case '8':	/* DECRC */
-		memcpy(&ictx->cell, &ictx->saved_cell, sizeof ictx->cell);
-		screen_write_cursormove(
-		    &ictx->ctx, ictx->saved_cx, ictx->saved_cy);
+	case '8':
+		switch (ictx->intermediate) {
+		case '\0':	/* DECRC */
+			memcpy(
+			    &ictx->cell, &ictx->saved_cell, sizeof ictx->cell);
+			screen_write_cursormove(
+			    &ictx->ctx, ictx->saved_cx, ictx->saved_cy);
+			break;
+		case '#':	/* DECALN */
+			screen_write_alignmenttest(&ictx->ctx);
+			break;
+		}
 		break;
 	default:
 		log_debug("unknown p2: %hhu", ch);
@@ -691,7 +745,7 @@ input_handle_standard_two(u_char ch, struct input_ctx *ictx)
 	    "-- s2 %zu: %hhu (%c) %hhu", ictx->off, ch, ch, ictx->intermediate);
 
 	switch (ch) {
-	case 'B':	/* Dscs (ASCII) */
+	case 'B':	/* SCS */
 		/*
 		 * Not really supported, but fake it up enough for those that
 		 * use it to switch character sets (by redefining G0 to
@@ -710,6 +764,8 @@ input_handle_standard_two(u_char ch, struct input_ctx *ictx)
 		ictx->saved_cx = 0;
 		ictx->saved_cy = 0;
 
+		screen_reset_tabs(ictx->ctx.s);
+
 		screen_write_scrollregion(
 		    &ictx->ctx, 0, screen_size_y(ictx->ctx.s) - 1);
 
@@ -736,7 +792,7 @@ input_handle_sequence(u_char ch, struct input_ctx *ictx)
 {
 	struct input_sequence_entry	*entry, find;
 	struct screen	 		*s = ictx->ctx.s;
-	u_int			         i;
+	u_int				 i;
 	struct input_arg 		*iarg;
 
 	log_debug2("-- sq %zu: %hhu (%c): %u [sx=%u, sy=%u, cx=%u, cy=%u, "
@@ -849,6 +905,48 @@ input_handle_sequence_dch(struct input_ctx *ictx)
 	screen_write_deletecharacter(&ictx->ctx, n);
 }
 
+void
+input_handle_sequence_cbt(struct input_ctx *ictx)
+{
+	struct screen  *s = ictx->ctx.s;
+	uint16_t	n;
+
+	if (ictx->private != '\0')
+		return;
+
+	if (ARRAY_LENGTH(&ictx->args) > 1)
+		return;
+	if (input_get_argument(ictx, 0, &n, 1) != 0)
+		return;
+	if (n == 0)
+		n = 1;
+
+	/* Find the previous tab point, n times. */
+	while (s->cx > 0 && n-- > 0) {
+		do
+			s->cx--;
+		while (s->cx > 0 && !bit_test(s->tabs, s->cx));
+	}
+}
+
+void
+input_handle_sequence_da(struct input_ctx *ictx)
+{
+	uint16_t	n;
+
+	if (ictx->private != '\0')
+		return;
+
+	if (ARRAY_LENGTH(&ictx->args) > 1)
+		return;
+	if (input_get_argument(ictx, 0, &n, 0) != 0)
+		return;
+	if (n != 0)
+		return;
+	
+	buffer_write(ictx->wp->out, "\033[?1;2c", (sizeof "\033[?1;2c") - 1);
+}
+
 void
 input_handle_sequence_dl(struct input_ctx *ictx)
 {
@@ -963,6 +1061,31 @@ input_handle_sequence_cup(struct input_ctx *ictx)
 	screen_write_cursormove(&ictx->ctx, m - 1, n - 1);
 }
 
+void
+input_handle_sequence_tbc(struct input_ctx *ictx)
+{
+	struct screen  *s = ictx->ctx.s;
+	uint16_t	n;
+
+	if (ictx->private != '\0')
+		return;
+
+	if (ARRAY_LENGTH(&ictx->args) > 1)
+		return;
+	if (input_get_argument(ictx, 0, &n, 1) != 0)
+		return;
+
+	switch (n) {
+	case 0:
+		if (s->cx < screen_size_x(s))
+			bit_clear(s->tabs, s->cx);
+		break;
+	case 3:
+		bit_nclear(s->tabs, 0, screen_size_x(s) - 1);
+		break;
+	}
+}
+
 void
 input_handle_sequence_ed(struct input_ctx *ictx)
 {
@@ -1022,7 +1145,10 @@ input_handle_sequence_el(struct input_ctx *ictx)
 void
 input_handle_sequence_sm(struct input_ctx *ictx)
 {
-	uint16_t	n;
+	struct window_pane	*wp = ictx->wp;
+	struct screen		*s = &wp->base;
+	u_int			 sx, sy;
+	uint16_t		 n;
 
 	if (ARRAY_LENGTH(&ictx->args) > 1)
 		return;
@@ -1035,6 +1161,10 @@ input_handle_sequence_sm(struct input_ctx *ictx)
 			screen_write_kcursormode(&ictx->ctx, 1);
 			log_debug("kcursor on");
 			break;
+		case 3:		/* DECCOLM */
+			screen_write_cursormove(&ictx->ctx, 0, 0);			
+			screen_write_clearscreen(&ictx->ctx);
+			break;
 		case 25:	/* TCEM */
 			screen_write_cursormode(&ictx->ctx, 1);
 			log_debug("cursor on");
@@ -1043,6 +1173,31 @@ input_handle_sequence_sm(struct input_ctx *ictx)
 			screen_write_mousemode(&ictx->ctx, 1);
 			log_debug("mouse on");
 			break;
+		case 1049:
+			if (wp->saved_grid != NULL)
+				break;
+			sx = screen_size_x(s);
+			sy = screen_size_y(s);
+
+			/*
+			 * Enter alternative screen mode. A copy of the visible
+			 * screen is saved and the history is not updated
+			 */
+
+			wp->saved_grid = grid_create(sx, sy, 0);
+			grid_duplicate_lines(
+			    wp->saved_grid, 0, s->grid, screen_hsize(s), sy);
+			wp->saved_cx = s->cx;
+			wp->saved_cy = s->cy;
+			memcpy(&wp->saved_cell,
+			    &ictx->cell, sizeof wp->saved_cell);
+
+			grid_view_clear(s->grid, 0, 0, sx, sy);
+
+			wp->base.grid->flags &= ~GRID_HISTORY;
+
+			wp->flags |= PANE_REDRAW;
+			break;
 		default:
 			log_debug("unknown SM [%hhu]: %u", ictx->private, n);
 			break;
@@ -1066,7 +1221,10 @@ input_handle_sequence_sm(struct input_ctx *ictx)
 void
 input_handle_sequence_rm(struct input_ctx *ictx)
 {
-	uint16_t	 n;
+	struct window_pane	*wp = ictx->wp;
+	struct screen		*s = &wp->base;
+	u_int			 sx, sy;
+	uint16_t		 n;
 
 	if (ARRAY_LENGTH(&ictx->args) > 1)
 		return;
@@ -1079,6 +1237,10 @@ input_handle_sequence_rm(struct input_ctx *ictx)
 			screen_write_kcursormode(&ictx->ctx, 0);
 			log_debug("kcursor off");
 			break;
+		case 3:		/* DECCOLM */
+			screen_write_cursormove(&ictx->ctx, 0, 0);			
+			screen_write_clearscreen(&ictx->ctx);
+			break;
 		case 25:	/* TCEM */
 			screen_write_cursormode(&ictx->ctx, 0);
 			log_debug("cursor off");
@@ -1087,6 +1249,48 @@ input_handle_sequence_rm(struct input_ctx *ictx)
 			screen_write_mousemode(&ictx->ctx, 0);
 			log_debug("mouse off");
 			break;
+		case 1049:
+			if (wp->saved_grid == NULL)
+				break;
+			sx = screen_size_x(s);
+			sy = screen_size_y(s);
+
+			/* 
+			 * Exit alternative screen mode and restore the copied
+			 * grid.
+			 */
+
+			/*
+			 * If the current size is bigger, temporarily resize
+			 * to the old size before copying back.
+			 */
+			if (sy > wp->saved_grid->sy)
+				screen_resize(s, sx, wp->saved_grid->sy);
+
+			/* Restore the grid, cursor position and cell. */
+			grid_duplicate_lines(
+			    s->grid, screen_hsize(s), wp->saved_grid, 0, sy);
+			s->cx = wp->saved_cx;
+			if (s->cx > screen_size_x(s) - 1)
+				s->cx = screen_size_x(s) - 1;
+			s->cy = wp->saved_cy;
+			if (s->cy > screen_size_y(s) - 1)
+				s->cy = screen_size_y(s) - 1;
+			memcpy(&ictx->cell, &wp->saved_cell, sizeof ictx->cell);
+
+			/*
+			 * Turn history back on (so resize can use it) and then
+			 * resize back to the current size.
+			 */
+  			wp->base.grid->flags |= GRID_HISTORY;
+			if (sy > wp->saved_grid->sy)
+				screen_resize(s, sx, sy);
+
+			grid_destroy(wp->saved_grid);
+			wp->saved_grid = NULL;
+
+			wp->flags |= PANE_REDRAW;
+			break;
 		default:
 			log_debug("unknown RM [%hhu]: %u", ictx->private, n);
 			break;
@@ -1129,7 +1333,6 @@ input_handle_sequence_dsr(struct input_ctx *ictx)
 			break;
 		}
 	}
-
 }
 
 void
@@ -1267,6 +1470,28 @@ input_handle_sequence_sgr(struct input_ctx *ictx)
 			gc->flags &= ~GRID_FLAG_BG256;
 			gc->bg = 8;
 			break;
+		case 90:
+		case 91:
+		case 92:
+		case 93:
+		case 94:
+		case 95:
+		case 96:
+		case 97:
+			gc->flags |= GRID_FLAG_FG256;
+			gc->fg = m - 82;
+			break;
+		case 100:
+		case 101:
+		case 102:
+		case 103:
+		case 104:
+		case 105:
+		case 106:
+		case 107:
+			gc->flags |= GRID_FLAG_BG256;
+			gc->bg = m - 92;
+			break;
 		}
 	}
 }
