@@ -1,4 +1,4 @@
-/* $Id: log.c,v 1.11 2009/03/31 21:22:10 nicm Exp $ */
+/* $Id: log.c,v 1.16 2009/07/22 17:31:20 tcunha Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -29,9 +29,8 @@
 
 /* Logging type. */
 #define LOG_TYPE_OFF 0
-#define LOG_TYPE_SYSLOG 1
-#define LOG_TYPE_TTY 2
-#define LOG_TYPE_FILE 3
+#define LOG_TYPE_TTY 1
+#define LOG_TYPE_FILE 2
 int	log_type = LOG_TYPE_OFF;
 
 /* Log file, if needed. */
@@ -40,17 +39,8 @@ FILE   *log_file;
 /* Debug level. */
 int	log_level;
 
-/* Open logging to syslog. */
-void
-log_open_syslog(int level)
-{
-	log_type = LOG_TYPE_SYSLOG;
-	log_level = level;
-
-	openlog(__progname, LOG_PID|LOG_NDELAY, LOG_FACILITY);
-
-	tzset();
-}
+void		 log_vwrite(int, const char *, va_list);
+__dead void	 log_vfatal(const char *, va_list);
 
 /* Open logging to tty. */
 void
@@ -91,17 +81,6 @@ log_close(void)
 	log_type = LOG_TYPE_OFF;
 }
 
-/* Write a log message. */
-void
-log_write(int pri, const char *msg, ...)
-{
-	va_list	ap;
-
-	va_start(ap, msg);
-	log_vwrite(pri, msg, ap);
-	va_end(ap);
-}
-
 /* Write a log message. */
 void
 log_vwrite(int pri, const char *msg, va_list ap)
@@ -110,13 +89,6 @@ log_vwrite(int pri, const char *msg, va_list ap)
 	FILE	*f = log_file;
 
 	switch (log_type) {
-	case LOG_TYPE_SYSLOG:
-#ifdef NO_VSYSLOG
-		/* XXX */
-#else
-		vsyslog(pri, msg, ap);
-#endif
-		break;
 	case LOG_TYPE_TTY:
 		if (pri == LOG_INFO)
 			f = stdout;
@@ -199,19 +171,6 @@ log_debug2(const char *msg, ...)
 	}
 }
 
-/* Log a debug message at level 3. */
-void printflike1
-log_debug3(const char *msg, ...)
-{
-	va_list	ap;
-
-	if (log_level > 2) {
-		va_start(ap, msg);
-		log_vwrite(LOG_DEBUG, msg, ap);
-		va_end(ap);
-	}
-}
-
 /* Log a critical error, with error string if necessary, and die. */
 __dead void
 log_vfatal(const char *msg, va_list ap)
