@@ -1,4 +1,4 @@
-/* $Id: names.c,v 1.4 2009/02/13 00:43:04 nicm Exp $ */
+/* $Id: names.c,v 1.17 2009/10/12 00:03:04 tcunha Exp $ */
 
 /*
  * Copyright (c) 2009 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -19,6 +19,7 @@
 #include <sys/types.h>
 
 #include <ctype.h>
+#include <libgen.h>
 #include <string.h>
 #include <unistd.h>
 
@@ -33,23 +34,24 @@ set_window_names(void)
 	u_int		 i;
 	char		*name, *wname;
 	struct timeval	 tv, tv2;
-	
+
 	if (gettimeofday(&tv, NULL) != 0)
-		fatal("gettimeofday");
+		fatal("gettimeofday failed");
 
 	for (i = 0; i < ARRAY_LENGTH(&windows); i++) {
 		w = ARRAY_ITEM(&windows, i);
 		if (w == NULL || w->active == NULL)
 			continue;
-		if (!options_get_number(&w->options, "automatic-rename"))
-			continue;
 
 		if (timercmp(&tv, &w->name_timer, <))
 			continue;
 		memcpy(&w->name_timer, &tv, sizeof w->name_timer);
 		tv2.tv_sec = 0;
 		tv2.tv_usec = NAME_INTERVAL * 1000L;
-		timeradd(&w->name_timer, &tv2, &w->name_timer);	
+		timeradd(&w->name_timer, &tv2, &w->name_timer);
+
+		if (!options_get_number(&w->options, "automatic-rename"))
+			continue;
 
 		if (w->active->screen != &w->active->base)
 			name = NULL;
@@ -58,10 +60,25 @@ set_window_names(void)
 		if (name == NULL)
 			wname = default_window_name(w);
 		else {
-			wname = parse_window_name(name);
+			/* 
+			 * If tmux is using the default command, it will be a
+			 * login shell and argv[0] may have a - prefix. Remove
+			 * this if it is present. Ick.
+			 */
+			if (w->active->cmd != NULL && *w->active->cmd == '\0' &&
+			    name != NULL && name[0] == '-' && name[1] != '\0')
+				wname = parse_window_name(name + 1);
+			else
+				wname = parse_window_name(name);
 			xfree(name);
 		}
 
+		if (w->active->fd == -1) {
+			xasprintf(&name, "%s[dead]", wname);
+			xfree(wname);
+			wname = name;
+		}
+
 		if (strcmp(wname, w->name) == 0)
 			xfree(wname);
 		else {
@@ -77,7 +94,9 @@ default_window_name(struct window *w)
 {
 	if (w->active->screen != &w->active->base)
 		return (xstrdup("[tmux]"));
-	return (parse_window_name(w->active->cmd));
+	if (w->active->cmd != NULL && *w->active->cmd != '\0')
+		return (parse_window_name(w->active->cmd));
+	return (parse_window_name(w->active->shell));
 }
 
 char *
@@ -88,7 +107,7 @@ parse_window_name(const char *in)
 	name = copy = xstrdup(in);
 	if (strncmp(name, "exec ", (sizeof "exec ") - 1) == 0)
 		name = name + (sizeof "exec ") - 1;
-	
+
 	while (*name == ' ')
 		name++;
 	if ((ptr = strchr(name, ' ')) != NULL)
@@ -96,14 +115,13 @@ parse_window_name(const char *in)
 
 	if (*name != '\0') {
 		ptr = name + strlen(name) - 1;
-		while (ptr > name && !isalnum(*ptr))
+		while (ptr > name && !isalnum((u_char)*ptr))
 			*ptr-- = '\0';
 	}
-	
+
 	if (*name == '/')
-		name = xbasename(name);
+		name = basename(name);
 	name = xstrdup(name);
 	xfree(copy);
 	return (name);
 }
-
