@@ -1,4 +1,4 @@
-/* $Id: osdep-openbsd.c,v 1.14 2009/02/17 18:54:14 nicm Exp $ */
+/* $Id: osdep-openbsd.c,v 1.19 2009/08/09 17:57:39 tcunha Exp $ */
 
 /*
  * Copyright (c) 2009 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -16,8 +16,6 @@
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-#ifdef __OpenBSD__
-
 #include <sys/param.h>
 #include <sys/sysctl.h>
 #include <sys/stat.h>
@@ -27,14 +25,64 @@
 #include <string.h>
 #include <unistd.h>
 
+#ifndef nitems
 #define nitems(_a) (sizeof((_a)) / sizeof((_a)[0]))
+#endif
 
 #define is_runnable(p) \
 	((p)->p_stat == SRUN || (p)->p_stat == SIDL || (p)->p_stat == SONPROC)
 #define is_stopped(p) \
 	((p)->p_stat == SSTOP || (p)->p_stat == SZOMB || (p)->p_stat == SDEAD)
 
-char	*osdep_get_name(int, char *);
+struct proc	*cmp_procs(struct proc *, struct proc *);
+char		*osdep_get_name(int, char *);
+
+struct proc *
+cmp_procs(struct proc *p1, struct proc *p2)
+{
+	void	*ptr1, *ptr2;
+
+	if (is_runnable(p1) && !is_runnable(p2))
+		return (p1);
+	if (!is_runnable(p1) && is_runnable(p2))
+		return (p2);
+
+	if (is_stopped(p1) && !is_stopped(p2))
+		return (p1);
+	if (!is_stopped(p1) && is_stopped(p2))
+		return (p2);
+
+	if (p1->p_estcpu > p2->p_estcpu)
+		return (p1);
+	if (p1->p_estcpu < p2->p_estcpu)
+		return (p2);
+
+	if (p1->p_slptime < p2->p_slptime)
+		return (p1);
+	if (p1->p_slptime > p2->p_slptime)
+		return (p2);
+
+	if ((p1->p_flag & P_SINTR) && !(p2->p_flag & P_SINTR))
+		return (p1);
+	if (!(p1->p_flag & P_SINTR) && (p2->p_flag & P_SINTR))
+		return (p2);
+
+	ptr1 = LIST_FIRST(&p1->p_children);
+	ptr2 = LIST_FIRST(&p2->p_children);
+	if (ptr1 == NULL && ptr2 != NULL)
+		return (p1);
+	if (ptr1 != NULL && ptr2 == NULL)
+		return (p2);
+
+	if (strcmp(p1->p_comm, p2->p_comm) < 0)
+		return (p1);
+	if (strcmp(p1->p_comm, p2->p_comm) > 0)
+		return (p2);
+
+	if (p1->p_pid > p2->p_pid)
+		return (p1);
+	return (p2);
+}
 
 char *
 osdep_get_name(int fd, char *tty)
@@ -43,7 +91,7 @@ osdep_get_name(int fd, char *tty)
 	struct stat	 sb;
 	size_t		 len;
 	struct kinfo_proc *buf, *newbuf;
-	struct proc	*p, *bestp;
+	struct proc	*bestp;
 	u_int		 i;
 	char		*name;
 
@@ -59,68 +107,24 @@ retry:
 		return (NULL);
 	len = (len * 5) / 4;
 
-	if ((newbuf = realloc(buf, len)) == NULL) {
-		free(buf);
-		return (NULL);
-	}
+	if ((newbuf = realloc(buf, len)) == NULL)
+		goto error;
 	buf = newbuf;
 
 	if (sysctl(mib, nitems(mib), buf, &len, NULL, 0) == -1) {
 		if (errno == ENOMEM)
 			goto retry;
-		free(buf);
-		return (NULL);
+		goto error;
 	}
 
 	bestp = NULL;
 	for (i = 0; i < len / sizeof (struct kinfo_proc); i++) {
 		if (buf[i].kp_eproc.e_tdev != sb.st_rdev)
 			continue;
-		p = &buf[i].kp_proc;
-		if (bestp == NULL) {
-			bestp = p;
-			continue;
-		}
-
-		if (is_runnable(p) && !is_runnable(bestp))
-			bestp = p;
-		else if (!is_runnable(p) && is_runnable(bestp))
-			continue;
-
-		if (!is_stopped(p) && is_stopped(bestp))
-			bestp = p;
-		else if (is_stopped(p) && !is_stopped(bestp))
-			continue;
-
-		if (p->p_estcpu > bestp->p_estcpu)
-			bestp = p;
-		else if (p->p_estcpu < bestp->p_estcpu)
-			continue;
-
-		if (p->p_slptime < bestp->p_slptime)
-			bestp = p;
-		else if (p->p_slptime > bestp->p_slptime)
-			continue;
-
-		if (p->p_flag & P_SINTR && !(bestp->p_flag & P_SINTR))
-			bestp = p;
-		else if (!(p->p_flag & P_SINTR) && bestp->p_flag & P_SINTR)
-			continue;
-
-		if (LIST_FIRST(&p->p_children) == NULL &&
-		    LIST_FIRST(&bestp->p_children) != NULL) /* XXX ugh */
-			bestp = p;
-		else if (LIST_FIRST(&p->p_children) != NULL &&
-		    LIST_FIRST(&bestp->p_children) == NULL)
-			continue;
-
-		if (strcmp(p->p_comm, p->p_comm) < 0)
-			bestp = p;
-		else if (strcmp(p->p_comm, p->p_comm) > 0)
-			continue;
-
-		if (p->p_pid > bestp->p_pid)
-			bestp = p;
+		if (bestp == NULL)
+			bestp = &buf[i].kp_proc;
+		else
+			bestp = cmp_procs(&buf[i].kp_proc, bestp);
 	}
 
 	name = NULL;
@@ -129,6 +133,8 @@ retry:
 
 	free(buf);
 	return (name);
-}
 
-#endif
+error:
+	free(buf);
+	return (NULL);
+}
