@@ -1,4 +1,4 @@
-/* $Id: screen.c,v 1.81 2009/03/28 20:17:29 nicm Exp $ */
+/* $Id: screen.c,v 1.96 2009/08/09 17:28:23 tcunha Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -18,6 +18,7 @@
 
 #include <sys/types.h>
 
+#include <stdlib.h>
 #include <string.h>
 
 #include "tmux.h"
@@ -33,6 +34,8 @@ screen_init(struct screen *s, u_int sx, u_int sy, u_int hlimit)
 
 	s->title = xstrdup("");
 
+	s->tabs = NULL;
+
 	screen_reinit(s);
 }
 
@@ -47,8 +50,10 @@ screen_reinit(struct screen *s)
 	s->rlower = screen_size_y(s) - 1;
 
 	s->mode = MODE_CURSOR;
+	
+	screen_reset_tabs(s);
 
-	grid_clear_lines(s->grid, s->grid->hsize, s->grid->sy - 1);
+	grid_clear_lines(s->grid, s->grid->hsize, s->grid->sy);
 
 	screen_clear_selection(s);
 }
@@ -57,16 +62,37 @@ screen_reinit(struct screen *s)
 void
 screen_free(struct screen *s)
 {
+	if (s->tabs != NULL)
+		xfree(s->tabs);
 	xfree(s->title);
 	grid_destroy(s->grid);
 }
 
+/* Reset tabs to default, eight spaces apart. */
+void
+screen_reset_tabs(struct screen *s)
+{
+	u_int	i;
+
+	if (s->tabs != NULL)
+		xfree(s->tabs);
+
+	if ((s->tabs = bit_alloc(screen_size_x(s))) == NULL)
+		fatal("bit_alloc failed");
+	for (i = 8; i < screen_size_x(s); i += 8)
+		bit_set(s->tabs, i);
+}
+
 /* Set screen title. */
 void
 screen_set_title(struct screen *s, const char *title)
 {
+	char	tmp[BUFSIZ];
+
+	strnvis(tmp, title, sizeof tmp, VIS_OCTAL|VIS_TAB|VIS_NL);
+
 	xfree(s->title);
-	s->title = xstrdup(title);
+	s->title = xstrdup(tmp);
 }
 
 /* Resize screen. */
@@ -78,8 +104,17 @@ screen_resize(struct screen *s, u_int sx, u_int sy)
 	if (sy < 1)
 		sy = 1;
 
-	if (sx != screen_size_x(s))
+	if (sx != screen_size_x(s)) {
 		screen_resize_x(s, sx);
+
+		/*
+		 * It is unclear what should happen to tabs on resize. xterm
+		 * seems to try and maintain them, rxvt resets them. Resetting
+		 * is simpler and more reliable so let's do that.
+		 */
+		screen_reset_tabs(s);
+	}
+
 	if (sy != screen_size_y(s))
 		screen_resize_y(s, sy);
 }
@@ -88,44 +123,19 @@ void
 screen_resize_x(struct screen *s, u_int sx)
 {
 	struct grid		*gd = s->grid;
-	const struct grid_cell	*gc;
-	const struct grid_utf8	*gu;
-	u_int			 xx, yy;
 
 	if (sx == 0)
 		fatalx("zero size");
 
-	/* If getting larger, not much to do. */
-	if (sx > screen_size_x(s)) {
-		gd->sx = sx;
-		return;
-	}
-
-	/* If getting smaller, nuke any data in lines over the new size. */
-	for (yy = gd->hsize; yy < gd->hsize + screen_size_y(s); yy++) {
-		/*
-		 * If the character after the last is wide or padding, remove
-		 * it and any leading padding.
-		 */
-		gc = &grid_default_cell;
-		for (xx = sx; xx > 0; xx--) {
-			gc = grid_peek_cell(gd, xx - 1, yy);
-			if (!(gc->flags & GRID_FLAG_PADDING))
-				break;
-			grid_set_cell(gd, xx - 1, yy, &grid_default_cell);
-		}
-		if (xx > 0 && xx != sx && gc->flags & GRID_FLAG_UTF8) {
-			gu = grid_peek_utf8(gd, xx - 1, yy);
-			if (gu->width > 1) {
-				grid_set_cell(
-				    gd, xx - 1, yy, &grid_default_cell);
-			}
-		}
-
-		/* Reduce the line size. */
-		grid_reduce_line(gd, yy, sx);
-	}
-
+	/*
+	 * Treat resizing horizontally simply: just ensure the cursor is
+	 * on-screen and change the size. Don't bother to truncate any lines -
+	 * then the data should be accessible if the size is then incrased.
+	 *
+	 * The only potential wrinkle is if UTF-8 double-width characters are
+	 * left in the last column, but UTF-8 terminals should deal with this
+	 * sanely.
+	 */
 	if (s->cx >= sx)
 		s->cx = sx - 1;
 	gd->sx = sx;
@@ -135,58 +145,82 @@ void
 screen_resize_y(struct screen *s, u_int sy)
 {
 	struct grid	*gd = s->grid;
-	u_int		 oy, yy, ny;
+	u_int		 needed, available, oldy, i;
 
 	if (sy == 0)
 		fatalx("zero size");
+	oldy = screen_size_y(s);
+
+	/* 
+	 * When resizing:
+	 *
+	 * If the height is decreasing, delete lines from the bottom until
+	 * hitting the cursor, then push lines from the top into the history.
+	 * 
+	 * When increasing, pull as many lines as possible from the history to
+	 * the top, then fill the remaining with blanks at the bottom.
+	 */
 
 	/* Size decreasing. */
-	if (sy < screen_size_y(s)) {
-		oy = screen_size_y(s);
+	if (sy < oldy) {
+		needed = oldy - sy;
 
-		if (s->cy != 0) {
-			/*
-			 * The cursor is not at the start. Try to remove as
-			 * many lines as possible from the top. (Up to the
-			 * cursor line.)
-			 */
-			ny = s->cy;
-			if (ny > oy - sy)
-				ny = oy - sy;
-
-			grid_view_delete_lines(gd, 0, ny);
-
- 			s->cy -= ny;
-			oy -= ny;
+		/* Delete as many lines as possible from the bottom. */
+		available = oldy - 1 - s->cy;
+		if (available > 0) {
+			if (available > needed)
+				available = needed;
+			grid_view_delete_lines(gd, oldy - available, available);
 		}
+		needed -= available;
 
-		if (sy < oy) {
-			/* Remove any remaining lines from the bottom. */
-			grid_view_delete_lines(gd, sy, oy - sy);
-			if (s->cy >= sy)
-				s->cy = sy - 1;
+		/*
+		 * Now just increase the history size, if possible, to take
+		 * over the lines which are left. If history is off, delete
+		 * lines from the top.
+		 *
+		 * XXX Should apply history limit?
+		 */
+		available = s->cy;
+		if (gd->flags & GRID_HISTORY)
+			gd->hsize += needed;
+		else if (needed > 0 && available > 0) {
+			if (available > needed)
+				available = needed;
+			grid_view_delete_lines(gd, 0, available);
 		}
-	}
+		s->cy -= needed;
+ 	}
 
 	/* Resize line arrays. */
-	gd->size = xrealloc(gd->size, gd->hsize + sy, sizeof *gd->size);
-	gd->data = xrealloc(gd->data, gd->hsize + sy, sizeof *gd->data);
-	gd->usize = xrealloc(gd->usize, gd->hsize + sy, sizeof *gd->usize);
-	gd->udata = xrealloc(gd->udata, gd->hsize + sy, sizeof *gd->udata);
+	gd->linedata = xrealloc(
+	    gd->linedata, gd->hsize + sy, sizeof *gd->linedata);
 
 	/* Size increasing. */
-	if (sy > screen_size_y(s)) {
-		oy = screen_size_y(s);
-		for (yy = gd->hsize + oy; yy < gd->hsize + sy; yy++) {
-			gd->size[yy] = 0;
-			gd->data[yy] = NULL;
-			gd->usize[yy] = 0;
-			gd->udata[yy] = NULL;			
-		}
+	if (sy > oldy) {
+		needed = sy - oldy;
+
+		/*
+		 * Try to pull as much as possible out of the history, if is
+		 * is enabled.
+		 */
+		available = gd->hsize;
+		if (gd->flags & GRID_HISTORY && available > 0) {
+			if (available > needed)
+				available = needed;
+			gd->hsize -= available;
+			s->cy += available;
+		} else
+			available = 0;
+		needed -= available;
+
+		/* Then fill the rest in with blanks. */
+		for (i = gd->hsize + sy - needed; i < gd->hsize + sy; i++)
+			memset(&gd->linedata[i], 0, sizeof gd->linedata[i]);
 	}
 
+	/* Set the new size, and reset the scroll region. */
 	gd->sy = sy;
-
 	s->rupper = 0;
 	s->rlower = screen_size_y(s) - 1;
 }
