@@ -1,4 +1,4 @@
-/* $Id: window-choose.c,v 1.13 2009/03/02 17:01:00 nicm Exp $ */
+/* $Id: window-choose.c,v 1.24 2009/10/12 00:18:19 tcunha Exp $ */
 
 /*
  * Copyright (c) 2009 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -27,7 +27,7 @@ void	window_choose_free(struct window_pane *);
 void	window_choose_resize(struct window_pane *, u_int, u_int);
 void	window_choose_key(struct window_pane *, struct client *, int);
 void	window_choose_mouse(
-    	    struct window_pane *, struct client *, u_char, u_char, u_char);
+    	    struct window_pane *, struct client *, struct mouse_event *);
 
 void	window_choose_redraw_screen(struct window_pane *);
 void	window_choose_write_line(
@@ -59,7 +59,8 @@ struct window_choose_mode_data {
 	u_int			top;
 	u_int			selected;
 
-	void 			(*callback)(void *, int);
+	void 			(*callbackfn)(void *, int);
+	void			(*freefn)(void *);
 	void		       *data;
 };
 
@@ -86,8 +87,8 @@ window_choose_add(struct window_pane *wp, int idx, const char *fmt, ...)
 }
 
 void
-window_choose_ready(struct window_pane *wp,
-    u_int cur, void (*callback)(void *, int), void *cdata)
+window_choose_ready(struct window_pane *wp, u_int cur,
+    void (*callbackfn)(void *, int), void (*freefn)(void *), void *cdata)
 {
 	struct window_choose_mode_data	*data = wp->modedata;
 	struct screen			*s = &data->screen;
@@ -96,7 +97,8 @@ window_choose_ready(struct window_pane *wp,
 	if (data->selected > screen_size_y(s) - 1)
 		data->top = ARRAY_LENGTH(&data->list) - screen_size_y(s);
 
-	data->callback = callback;
+	data->callbackfn = callbackfn;
+	data->freefn = freefn;
 	data->data = cdata;
 
 	window_choose_redraw_screen(wp);
@@ -107,21 +109,29 @@ window_choose_init(struct window_pane *wp)
 {
 	struct window_choose_mode_data	*data;
 	struct screen			*s;
+	int				 keys;
 
 	wp->modedata = data = xmalloc(sizeof *data);
-	data->callback = NULL;
+
+	data->callbackfn = NULL;
+	data->freefn = NULL;
+	data->data = NULL;
+
 	ARRAY_INIT(&data->list);
 	data->top = 0;
 
 	s = &data->screen;
 	screen_init(s, screen_size_x(&wp->base), screen_size_y(&wp->base), 0);
 	s->mode &= ~MODE_CURSOR;
-	s->mode |= MODE_MOUSE;
+	if (options_get_number(&wp->window->options, "mode-mouse"))
+		s->mode |= MODE_MOUSE;
+
+	keys = options_get_number(&wp->window->options, "mode-keys");
+	if (keys == MODEKEY_EMACS)
+		mode_key_init(&data->mdata, &mode_key_tree_emacs_choice);
+	else
+		mode_key_init(&data->mdata, &mode_key_tree_vi_choice);
 
-	mode_key_init(&data->mdata,
-	    options_get_number(&wp->window->options, "mode-keys"),
-	    MODEKEY_CHOOSEMODE);
-	
 	return (s);
 }
 
@@ -131,8 +141,9 @@ window_choose_free(struct window_pane *wp)
 	struct window_choose_mode_data	*data = wp->modedata;
 	u_int				 i;
 
- 	mode_key_free(&data->mdata);
-	
+	if (data->freefn != NULL && data->data != NULL)
+		data->freefn(data->data);
+
 	for (i = 0; i < ARRAY_LENGTH(&data->list); i++)
 		xfree(ARRAY_ITEM(&data->list, i).name);
 	ARRAY_FREE(&data->list);
@@ -167,16 +178,16 @@ window_choose_key(struct window_pane *wp, unused struct client *c, int key)
 	items = ARRAY_LENGTH(&data->list);
 
 	switch (mode_key_lookup(&data->mdata, key)) {
-	case MODEKEYCMD_QUIT:
-		data->callback(data->data, -1);
+	case MODEKEYCHOICE_CANCEL:
+		data->callbackfn(data->data, -1);
 		window_pane_reset_mode(wp);
 		break;
-	case MODEKEYCMD_CHOOSE:
+	case MODEKEYCHOICE_CHOOSE:
 		item = &ARRAY_ITEM(&data->list, data->selected);
-		data->callback(data->data, item->idx);
+		data->callbackfn(data->data, item->idx);
 		window_pane_reset_mode(wp);
 		break;
-	case MODEKEYCMD_UP:
+	case MODEKEYCHOICE_UP:
 		if (items == 0)
 			break;
 		if (data->selected == 0) {
@@ -196,9 +207,9 @@ window_choose_key(struct window_pane *wp, unused struct client *c, int key)
 			window_choose_write_line(
 			    wp, &ctx, data->selected + 1 - data->top);
 			screen_write_stop(&ctx);
-		}			
+		}
 		break;
-	case MODEKEYCMD_DOWN:
+	case MODEKEYCHOICE_DOWN:
 		if (items == 0)
 			break;
 		if (data->selected == items - 1) {
@@ -208,6 +219,7 @@ window_choose_key(struct window_pane *wp, unused struct client *c, int key)
 			break;
 		}
 		data->selected++;
+
 		if (data->selected >= data->top + screen_size_y(&data->screen))
 			window_choose_scroll_down(wp);
 		else {
@@ -217,9 +229,9 @@ window_choose_key(struct window_pane *wp, unused struct client *c, int key)
 			window_choose_write_line(
 			    wp, &ctx, data->selected - 1 - data->top);
 			screen_write_stop(&ctx);
-		}			
+		}
 		break;
-	case MODEKEYCMD_PREVIOUSPAGE:
+	case MODEKEYCHOICE_PAGEUP:
 		if (data->selected < screen_size_y(s)) {
 			data->selected = 0;
 			data->top = 0;
@@ -232,7 +244,7 @@ window_choose_key(struct window_pane *wp, unused struct client *c, int key)
 		}
  		window_choose_redraw_screen(wp);
 		break;
-	case MODEKEYCMD_NEXTPAGE:
+	case MODEKEYCHOICE_PAGEDOWN:
 		data->selected += screen_size_y(s);
 		if (data->selected > items - 1)
 			data->selected = items - 1;
@@ -252,28 +264,28 @@ window_choose_key(struct window_pane *wp, unused struct client *c, int key)
 }
 
 void
-window_choose_mouse(struct window_pane *wp,
-    unused struct client *c, u_char b, u_char x, u_char y)
+window_choose_mouse(
+    struct window_pane *wp, unused struct client *c, struct mouse_event *m)
 {
 	struct window_choose_mode_data	*data = wp->modedata;
 	struct screen			*s = &data->screen;
 	struct window_choose_mode_item	*item;
 	u_int				 idx;
 
-	if ((b & 3) == 3)
+	if ((m->b & 3) == 3)
 		return;
-	if (x >= screen_size_x(s))
+	if (m->x >= screen_size_x(s))
 		return;
-	if (y >= screen_size_y(s))
+	if (m->y >= screen_size_y(s))
 		return;
 
-	idx = data->top + y;
+	idx = data->top + m->y;
 	if (idx >= ARRAY_LENGTH(&data->list))
 		return;
 	data->selected = idx;
 
 	item = &ARRAY_ITEM(&data->list, data->selected);
-	data->callback(data->data, item->idx);
+	data->callbackfn(data->data, item->idx);
 	window_pane_reset_mode(wp);
 }
 
@@ -283,28 +295,30 @@ window_choose_write_line(
 {
 	struct window_choose_mode_data	*data = wp->modedata;
 	struct window_choose_mode_item	*item;
+	struct options			*oo = &wp->window->options;
 	struct screen			*s = &data->screen;
 	struct grid_cell		 gc;
+ 	int				 utf8flag;
 
-	if (data->callback == NULL)
+	if (data->callbackfn == NULL)
 		fatalx("called before callback assigned");
 
+	utf8flag = options_get_number(&wp->window->options, "utf8");
 	memcpy(&gc, &grid_default_cell, sizeof gc);
 	if (data->selected == data->top + py) {
-		gc.fg = options_get_number(&wp->window->options, "mode-bg");
-		gc.bg = options_get_number(&wp->window->options, "mode-fg");
-		gc.attr |= options_get_number(&wp->window->options, "mode-attr");
+		colour_set_fg(&gc, options_get_number(oo, "mode-fg"));
+		colour_set_bg(&gc, options_get_number(oo, "mode-bg"));
+		gc.attr |= options_get_number(oo, "mode-attr");
 	}
 
 	screen_write_cursormove(ctx, 0, py);
 	if (data->top + py  < ARRAY_LENGTH(&data->list)) {
 		item = &ARRAY_ITEM(&data->list, data->top + py);
-		screen_write_puts(
-		    ctx, &gc, "%.*s", (int) screen_size_x(s), item->name);
+		screen_write_nputs(
+		    ctx, screen_size_x(s) - 1, &gc, utf8flag, "%s", item->name);
 	}
 	while (s->cx < screen_size_x(s))
 		screen_write_putc(ctx, &gc, ' ');
-
 }
 
 void
@@ -354,7 +368,7 @@ window_choose_scroll_down(struct window_pane *wp)
 	screen_write_start(&ctx, wp, NULL);
 	screen_write_cursormove(&ctx, 0, 0);
 	screen_write_deleteline(&ctx, 1);
-	window_choose_write_line(wp, &ctx, screen_size_y(s) - 1);	
+	window_choose_write_line(wp, &ctx, screen_size_y(s) - 1);
 	if (screen_size_y(&data->screen) > 1)
 		window_choose_write_line(wp, &ctx, screen_size_y(s) - 2);
 	screen_write_stop(&ctx);
